﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Operators {
  using CloneMapType = HeuristicLab.Core.ItemDictionary<HeuristicLab.Core.IItem, HeuristicLab.Core.IItem>;
  using TraceMapType = HeuristicLab.Core.ItemDictionary<HeuristicLab.Core.IItem, HeuristicLab.Core.IItemList<HeuristicLab.Core.IItem>>;

  /// <summary>
  /// An operator that creates the data structures needed for algorithm bevahior analysis.
  /// </summary>
  [Item("CreateTrackingDatastructures", "An operator that creates the data structures needed for algorithm bevahior analysis.")]
  [StorableClass]
  public class CreateTrackingDatastructures : SingleSuccessorOperator {
    public const string ParentsParameterName = "Parents";
    public const string ChildParameterName = "Child";
    public const string GlobalTraceMapParameterName = "GlobalTraceMap";
    public const string GlobalCloneMapParameterName = "GlobalCloneMap";

    #region Parameter properties
    public IValueParameter<CloneMapType> GlobalCloneMapParameter {
      get { return (ValueParameter<CloneMapType>)Parameters[GlobalCloneMapParameterName]; }
    }
    public IValueParameter<TraceMapType> GlobalTraceMapParameter {
      get { return (ValueParameter<TraceMapType>)Parameters[GlobalTraceMapParameterName]; }
    }
    #endregion

    #region Properties
    public CloneMapType GlobalCloneMap {
      get { return GlobalCloneMapParameter.Value; }
    }
    public TraceMapType GlobalTraceMap {
      get { return GlobalTraceMapParameter.Value; }
    }
    #endregion

    [StorableConstructor]
    private CreateTrackingDatastructures(bool deserializing) : base(deserializing) { }
    private CreateTrackingDatastructures(CreateTrackingDatastructures original, Cloner cloner) : base(original, cloner) { }
    public CreateTrackingDatastructures()
      : base() {
      Parameters.Add(new ValueParameter<CloneMapType>(GlobalCloneMapParameterName, "A global map keeping track of trees and their clones (made during selection).", new CloneMapType()));
      Parameters.Add(new ValueParameter<TraceMapType>(GlobalTraceMapParameterName, "A global cache containing tracing info.", new TraceMapType()));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CreateTrackingDatastructures(this, cloner);
    }

    public override IOperation Apply() {
      var gScope = ExecutionContext.Scope;
      while (gScope.Parent != null) {
        gScope = gScope.Parent;
      }

      gScope.Variables.Add(new Variable(GlobalCloneMapParameterName, GlobalCloneMap));
      gScope.Variables.Add(new Variable(GlobalTraceMapParameterName, GlobalTraceMap));

      return base.Apply();
    }
  }
}
